@@ -337,7 +337,8 @@ class Dataset:
 
     def prepare_labels(self):
         strategy = self.util.config_val('DATA', 'strategy', 'train_test')
-        if strategy == 'cross_data':
+        only_tests = eval(self.util.config_val('DATA', 'tests', 'False'))
+        if strategy == 'cross_data' or only_tests:
             self.df = self.map_labels(self.df)
             # Bin target values if they are continuous but a classification experiment should be done
             self.map_continuous_classification(self.df)
@@ -346,7 +347,7 @@ class Dataset:
                 if not self.got_gender or not self.got_speaker:
                     self.util.error('can\'t plot value counts if no speaker or gender is given')
                 else:
-                    self.plot.describe_df(self.name, self.df, self.target, f'{self.name}_distplot.png')
+                    self.plot.describe_df(self.name, self.df, self.target, f'{self.name}_distplot')
         elif strategy == 'train_test':        
             self.df_train = self.map_labels(self.df_train)
             self.df_test = self.map_labels(self.df_test)
@@ -358,8 +359,8 @@ class Dataset:
                 if not self.got_gender or not self.got_speaker:
                     self.util.error('can\'t plot value counts if no speaker or gender is given')
                 else:
-                    self.plot.describe_df(self.name, self.df_train, self.target, f'{self.name}_train_distplot.png')
-                    self.plot.describe_df(self.name, self.df_test, self.target, f'{self.name}_test_distplot.png')
+                    self.plot.describe_df(self.name, self.df_train, self.target, f'{self.name}_train_distplot')
+                    self.plot.describe_df(self.name, self.df_test, self.target, f'{self.name}_test_distplot')
 
 
     def map_labels(self, df):
@@ -376,8 +377,12 @@ class Dataset:
             df[target] = df[target].map(mapping)
             self.util.debug(f'{self.name}: mapped {mapping}')
         # remove labels that are not in the labels list
-        labels = ast.literal_eval(glob_conf.config['DATA']['labels'])
-        df = df[df[target].isin(labels)]
+        labels = self.util.config_val('DATA', 'labels', False)
+        if labels:
+            labels = ast.literal_eval(labels)
+            df = df[df[target].isin(labels)]
+        else:
+            labels = df[target].unique().codes
         # try:
         # except KeyError:
         #     pass
