@@ -8,6 +8,7 @@ from nkululeko.dataset_ravdess import Ravdess
 from nkululeko.filter_data import filter_min_dur
 from nkululeko.runmanager import Runmanager
 from nkululeko.test_predictor import Test_predictor
+from nkululeko.feats_analyser import FeatureAnalyser
 from nkululeko.util import Util
 from nkululeko.feature_extractor import FeatureExtractor
 from nkululeko.plots import Plots
@@ -82,6 +83,45 @@ class Experiment:
         df.is_labeled = True if self.target in df else False
         return df
 
+    def fill_tests(self):
+        """Only fill a new test set"""
+
+        test_dbs = ast.literal_eval(glob_conf.config['DATA']['tests'])
+        self.df_test = pd.DataFrame()
+        start_fresh = eval(self.util.config_val('DATA', 'no_reuse', 'False'))
+        store = self.util.get_path('store')
+        storage_test = f'{store}extra_testdf.csv'
+        if os.path.isfile(storage_test) \
+            and not start_fresh:
+            self.util.debug(f'reusing previously stored {storage_test}')
+            self.df_test = self._import_csv(storage_test)
+        else:
+            for d in test_dbs:
+                if d == 'ravdess':
+                    data = Ravdess()
+                else:
+                    ds_type = self.util.config_val_data(d, 'type', 'audformat')
+                    if ds_type == 'audformat':
+                        data = Dataset(d)
+                    elif ds_type == 'csv':
+                        data = Dataset_CSV(d)
+                    else:
+                        self.util.error(f'unknown data type: {ds_type}')
+                data.load()
+                if data.got_gender:
+                    self.got_gender = True
+                if data.got_speaker:
+                    self.got_speaker = True
+                data.prepare_labels()
+                self.df_test = pd.concat([self.df_test, self.util.make_segmented_index(data.df)])
+                self.df_test.is_labeled = data.is_labeled
+            self.df_test.got_gender = self.got_gender
+            self.df_test.got_speaker = self.got_speaker
+            # self.util.set_config_val('FEATS', 'needs_features_extraction', 'True')
+            # self.util.set_config_val('FEATS', 'no_reuse', 'True')
+            self.df_test['class_labels'] = self.df_test[self.target]
+            self.df_test[self.target] = self.label_encoder.transform(self.df_test[self.target])
+            self.df_test.to_csv(storage_test)
 
     def fill_train_and_tests(self):
         """Set up train and development sets. The method should be specified in the config."""
@@ -211,12 +251,12 @@ class Experiment:
             # if self.df_test.is_labeled:
             #     self.df_test['labels'] = self.label_encoder.inverse_transform(self.df_test[self.target])
             if self.df_test.shape[0] > 0:
-                plot.describe_df('dev_set', self.df_test, self.target, f'test_distplot.png')
-            plot.describe_df('train_set', self.df_train, self.target, f'train_distplot.png')
+                plot.describe_df('dev_set', self.df_test, self.target, f'test_distplot')
+            plot.describe_df('train_set', self.df_train, self.target, f'train_distplot')
         else:
             if self.df_test.shape[0] > 0:
-                plot.describe_df('dev_set', self.df_test, self.target, f'test_distplot.png')
-            plot.describe_df('train_set', self.df_train, self.target, f'train_distplot.png')
+                plot.describe_df('dev_set', self.df_test, self.target, f'test_distplot')
+            plot.describe_df('train_set', self.df_train, self.target, f'train_distplot')
 
 
     def augment_train(self):
@@ -226,6 +266,13 @@ class Experiment:
         df_train_aug = augment_train.augment()
         self.df_train = self.df_train.append(df_train_aug)
 
+    def extract_test_feats(self):
+        self.feats_test = pd.DataFrame()
+        feats_name = "_".join(ast.literal_eval(glob_conf.config['DATA']['tests']))
+        self.feature_extractor = FeatureExtractor() 
+        self.feature_extractor.set_data(self.df_test, feats_name, 'test')
+        self.feats_test = self.feature_extractor.extract()
+        self.util.debug(f'Test features shape:{self.feats_test.shape}')
 
     def extract_feats(self):
         """Extract the features for train and dev sets. 
@@ -236,31 +283,66 @@ class Experiment:
         
         """
         df_train, df_test = self.df_train, self.df_test
-        strategy = self.util.config_val('DATA', 'strategy', 'traintest')
-        feats_types = self.util.config_val_list('FEATS', 'type', ['os'])
         feats_name = "_".join(ast.literal_eval(glob_conf.config['DATA']['databases']))
         self.feats_test, self.feats_train = pd.DataFrame(), pd.DataFrame()
-        _scale = True
         self.feature_extractor = FeatureExtractor() 
-        # featExtractor_train = FeatureExtractor(df_train, feats_name, 'train')
-        # featExtractor_test = FeatureExtractor(df_test, feats_name, 'test')
         self.feature_extractor.set_data(df_train, feats_name, 'train')
         self.feats_train =self.feature_extractor.extract()
         self.feature_extractor.set_data(df_test, feats_name, 'test')
         self.feats_test = self.feature_extractor.extract()
         self.util.debug(f'All features: train shape : {self.feats_train.shape}, test shape:{self.feats_test.shape}')
-        if _scale:
-            self._scale()
 
         # check if a tsne should be plotted
-        tsne = self.util.config_val('PLOT', 'tsne', False)
-        if tsne and self.util.exp_is_classification():
-            plots = Plots()
-            all_feats =self.feats_train.append(self.feats_test)
-            all_labels = self.df_train['class_label'].append(self.df_test['class_label'])
-            plots.plotTsne(all_feats, all_labels, self.util.get_exp_name()+'_tsne')
+        tsne = eval(self.util.config_val('EXPL', 'tsne', 'False'))
+        if tsne: 
+            if self.util.exp_is_classification():
+                plots = Plots()
+                all_feats =self.feats_train.append(self.feats_test)
+                all_labels = self.df_train['class_label'].append(self.df_test['class_label'])
+                plots.plotTsne(all_feats, all_labels, self.util.get_exp_name()+'_tsne')
+            else:
+                 self.util.debug('can\'t plot tsne if not classification')
 
-    def _scale(self):
+        # check if feature distributions should be plotted
+        plot_feats = self.util.config_val('EXPL', 'feature_distributions', False)
+        if plot_feats: 
+            if self.util.exp_is_classification():
+                plots = Plots()
+                if plot_feats=='all':
+                    df_feats = pd.concat([self.feats_train, self.feats_test])
+                    df_labels = pd.concat([self.df_train, self.df_test])
+                elif plot_feats=='train':
+                    df_feats = self.feats_train
+                    df_labels = self.df_train
+                elif plot_feats=='test':
+                    df_feats = self.feats_test
+                    df_labels = self.df_test
+                else:
+                    self.util.error(f'unkown feature_distribution specifier {plot_feats}, should be [all | train | test]')
+                for feature in df_feats.columns:
+                    plots.plot_feature(plot_feats, feature, 'class_label', df_labels, df_feats)
+            else:
+                self.util.debug('can\'t plot feature distributions if not classification')
+        self._check_scale()
+
+    def analyse_features(self):
+        sample_selection = self.util.config_val('EXPL', 'sample_selection', 'all')
+        if sample_selection=='all':
+            df_feats = pd.concat([self.feats_train, self.feats_test])
+            df_labels = pd.concat([self.df_train, self.df_test])
+        elif sample_selection=='train':
+            df_feats = self.feats_train
+            df_labels = self.df_train
+        elif sample_selection=='test':
+            df_feats = self.feats_test
+            df_labels = self.df_test
+        else:
+            self.util.error(f'unkown feature_distribution specifier {sample_selection}, should be [all | train | test]')
+        feat_analyser = FeatureAnalyser(sample_selection, df_labels[self.target], df_feats)        
+        feat_analyser.analyse()
+
+
+    def _check_scale(self):
         scale = self.util.config_val('FEATS', 'scale', False)
         if scale: 
             self.scaler = Scaler(self.df_train, self.df_test, self.feats_train, self.feats_test, scale)
@@ -307,7 +389,7 @@ class Experiment:
 
     def plot_confmat_per_speaker(self, function):
         if self.loso or self.logo or self.xfoldx:
-            self.util.debug('plot combined speaker predictins not possible for cross validation')
+            self.util.debug('plot combined speaker predictions not possible for cross validation')
             return
         best = self._get_best_report(self.reports)
         # if not best.is_classification:
@@ -317,14 +399,13 @@ class Experiment:
         speakers = self.df_test.speaker.values
         print(f'{len(truths)} {len(preds)} {len(speakers) }')
         df = pd.DataFrame(data={'truth':truths, 'pred':preds, 'speaker':speakers})
-        plot_name = 'result_combined_per_speaker.png'
+        plot_name = 'result_combined_per_speaker'
         self.util.debug(f'plotting speaker combination ({function}) confusion matrix to {plot_name}')
         best.plot_per_speaker(df, plot_name, function)
 
     def _get_best_report(self, best_reports):
         return self.runmgr.get_best_result(best_reports)
 
-
     def print_best_model(self):
         self.runmgr.print_best_result_runs()
 
@@ -340,7 +421,7 @@ class Experiment:
 
     def predict_test_and_save(self, result_name):
         model = self.runmgr.get_best_model()
-
+        model.set_testdata(self.df_test, self.feats_test)
         test_predictor = Test_predictor(model, self.df_test, self.label_encoder, result_name)        
         test_predictor.predict_and_store()
 
